{
  "cells": [
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "# **\\[SOLAR\\]** SIMULATION (GConvLSTM, GCLSTM, LRGCN +)\n",
        "\n",
        "JiyunLim  \n",
        "2020-12-12\n",
        "\n",
        "## Data"
      ],
      "id": "5e80c314-4569-42a0-8316-d350c23e7289"
    },
    {
      "cell_type": "code",
      "execution_count": 1,
      "metadata": {},
      "outputs": [],
      "source": [
        "import pandas as pd\n",
        "import numpy as np\n",
        "import os"
      ],
      "id": "c05401f9-7015-4b1a-b52b-4df542c0efc6"
    },
    {
      "cell_type": "code",
      "execution_count": 3,
      "metadata": {},
      "outputs": [],
      "source": [
        "# read dataframe \n",
        "df = pd.read_csv('data_eng_230710.csv')\n",
        "\n",
        "# make y, y_upper, y_period, time, regions \n",
        "y = df.loc[:,'Bukchoncheon':'Gyeongju-si'].to_numpy()\n",
        "yU = df.loc[:,'Bukchoncheon_Upper':'Gyeongju-si_Upper'].to_numpy()\n",
        "yP = np.divide(y, yU+1e-10)\n",
        "t = df.loc[:,'date']\n",
        "regions = list(df.loc[:,'Bukchoncheon':'Gyeongju-si'].columns)\n",
        "\n",
        "TRAIN_RATIO = 0.8\n",
        "T,N = len(t),len(regions) \n",
        "LEN_TEST = int(np.floor(T*(1-TRAIN_RATIO)))\n",
        "LEN_TR = T - LEN_TEST"
      ],
      "id": "f0fed204-2db1-4334-a15e-e53bb3b5187c"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "## GConvLSTM / GCLSTM / LRGCN"
      ],
      "id": "0a97f73b-0b25-47cc-bc15-8fa37adda658"
    },
    {
      "cell_type": "code",
      "execution_count": 4,
      "metadata": {},
      "outputs": [],
      "source": [
        "%run model/lstm"
      ],
      "id": "c2ac17b9-8601-402c-8f8a-09ac2882f226"
    },
    {
      "cell_type": "code",
      "execution_count": 5,
      "metadata": {},
      "outputs": [],
      "source": [
        "s2 = Simulator()\n",
        "s3 = Simulator()\n",
        "s4 = Simulator()"
      ],
      "id": "e6a6d495-5497-4c39-b36e-b9933069b62f"
    },
    {
      "cell_type": "code",
      "execution_count": 6,
      "metadata": {},
      "outputs": [],
      "source": [
        "class GConv_LSTM(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(GConv_LSTM, self).__init__()\n",
        "        self.recurrent = GConvLSTM(in_channels = node_features, out_channels = filters, K = 2)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight, h, c):\n",
        "        h_0, c_0 = self.recurrent(x, edge_index, edge_weight, h, c)\n",
        "        h = F.relu(h_0)\n",
        "        h = self.linear(h)\n",
        "        return h, h_0, c_0"
      ],
      "id": "1e51e82e-6885-4c06-934d-2d535a45b330"
    },
    {
      "cell_type": "code",
      "execution_count": 7,
      "metadata": {},
      "outputs": [],
      "source": [
        "class GC_LSTM(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(GC_LSTM, self).__init__()\n",
        "        self.recurrent = GCLSTM(node_features, filters, 2)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight, h, c):\n",
        "        h_0, c_0 = self.recurrent(x, edge_index, edge_weight, h, c)\n",
        "        h = F.relu(h_0)\n",
        "        h = self.linear(h)\n",
        "        return h, h_0, c_0"
      ],
      "id": "a721a15c-16cf-4f4b-8f78-668cd1efbbe2"
    },
    {
      "cell_type": "code",
      "execution_count": 8,
      "metadata": {},
      "outputs": [],
      "source": [
        "class LR_GCN(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(LR_GCN, self).__init__()\n",
        "        self.recurrent = LRGCN(node_features, filters, 1, 1)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight, h_0, c_0):\n",
        "        h_0, c_0 = self.recurrent(x, edge_index, edge_weight, h_0, c_0)\n",
        "        h = F.relu(h_0)\n",
        "        h = self.linear(h)\n",
        "        return h, h_0, c_0"
      ],
      "id": "ac1d13e4-b9c2-463b-bb66-8de1b2cd36c9"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [
        {
          "output_type": "stream",
          "name": "stdout",
          "text": [
            "method:classic lag:4 filters:4 epoch:50 is done\n",
            "method:classic lag:4 filters:8 epoch:50 is done\n",
            "method:classic lag:4 filters:16 epoch:50 is done\n",
            "method:classic lag:4 filters:32 epoch:50 is done\n",
            "method:classic lag:8 filters:4 epoch:50 is done\n",
            "method:classic lag:8 filters:8 epoch:50 is done\n",
            "method:classic lag:8 filters:16 epoch:50 is done\n",
            "method:classic lag:8 filters:32 epoch:50 is done\n",
            "method:classic lag:12 filters:4 epoch:50 is done\n",
            "method:classic lag:12 filters:8 epoch:50 is done\n",
            "method:classic lag:12 filters:16 epoch:50 is done\n",
            "method:classic lag:12 filters:32 epoch:50 is done\n",
            "method:proposed lag:4 filters:4 epoch:50 is done\n",
            "method:proposed lag:4 filters:8 epoch:50 is done\n",
            "method:proposed lag:4 filters:16 epoch:50 is done\n",
            "method:proposed lag:4 filters:32 epoch:50 is done\n",
            "method:proposed lag:8 filters:4 epoch:50 is done\n",
            "method:proposed lag:8 filters:8 epoch:50 is done\n",
            "method:proposed lag:8 filters:16 epoch:50 is done\n",
            "method:proposed lag:12 filters:4 epoch:50 is done\n",
            "method:proposed lag:12 filters:8 epoch:50 is done\n",
            "method:proposed lag:12 filters:16 epoch:50 is done\n",
            "method:proposed lag:12 filters:32 epoch:50 is done\n",
            "save results\n",
            "method:classic lag:4 filters:4 epoch:50 is done\n",
            "38/50"
          ]
        }
      ],
      "source": [
        "s2.simulate(\n",
        "    n_iteration = 30,\n",
        "    lags = [4, 8, 12],\n",
        "    filters = [4, 8, 16, 32],\n",
        "    epoch = [50],\n",
        "    Model = GConv_LSTM\n",
        ")\n",
        "\n",
        "\n",
        "s3.simulate(\n",
        "    n_iteration = 30,\n",
        "    lags = [4, 8, 12],\n",
        "    filters = [4, 8, 16, 32],\n",
        "    epoch = [50],\n",
        "    Model = GC_LSTM\n",
        ")\n",
        "\n",
        "s4.simulate(\n",
        "    n_iteration = 30,\n",
        "    lags = [4, 8, 12],\n",
        "    filters = [4, 8, 16, 32],\n",
        "    epoch = [50],\n",
        "    Model = LR_GCN\n",
        ")"
      ],
      "id": "3653959f-efe7-4e32-8bb7-5e170f18264b"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "fnames = ['./results/'+l for l in os.listdir('./results/') if l[0] != '.']\n",
        "fnames "
      ],
      "id": "3ae8dc7b-3540-4adf-a707-9f7917503932"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "%run model/tgcn"
      ],
      "id": "2ec7583e-9435-4f8d-b9f8-20e5fb4eea7b"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s5 = Simulator()"
      ],
      "id": "2ed468c2-9ffb-4068-9e71-1d32f997f4bb"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "class T_GCN(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(T_GCN, self).__init__()\n",
        "        self.recurrent = TGCN(node_features, filters)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight, prev_hidden_state):\n",
        "        h = self.recurrent(x, edge_index, edge_weight, prev_hidden_state)\n",
        "        y = F.relu(h)\n",
        "        y = self.linear(y)\n",
        "        return y, h"
      ],
      "id": "26db9ac8-f1db-449d-91e7-b8c59137ecf7"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s5.simulate(\n",
        "    n_iteration = 1,\n",
        "    lags = [4, 8, 12],\n",
        "    filters = [4, 8, 16, 32],\n",
        "    epoch = [50],\n",
        "    Model = T_GCN\n",
        ")"
      ],
      "id": "f68c747f-60c1-4dba-a36a-39a20b8c9c8d"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "fnames = ['./results/'+l for l in os.listdir('./results/') if l[0] != '.']\n",
        "fnames "
      ],
      "id": "b12f5878-f102-4586-95e4-0de7e67fd53a"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "%run model/dcrnn"
      ],
      "id": "ef73061f-3172-4167-87dc-acd410ce5b9b"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s6 = Simulator()"
      ],
      "id": "d0a2d279-a68d-4624-9371-1b6c9cbab070"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "class DC_RNN(torch.nn.Module):\n",
        "    def __init__(self, node_features, filters):\n",
        "        super(DC_RNN, self).__init__()\n",
        "        self.recurrent = DCRNN(node_features, filters, 2)\n",
        "        self.linear = torch.nn.Linear(filters, 1)\n",
        "\n",
        "    def forward(self, x, edge_index, edge_weight):\n",
        "        h = self.recurrent(x, edge_index, edge_weight)\n",
        "        h = F.relu(h)\n",
        "        h = self.linear(h)\n",
        "        return h"
      ],
      "id": "0ccca810-3410-4f2b-a786-8b16700d3d1f"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "s6.simulate(\n",
        "    n_iteration = 1,\n",
        "    lags = [4, 8, 12],\n",
        "    filters = [4, 8, 16, 32],\n",
        "    epoch = [50],\n",
        "    Model = DC_RNN\n",
        ")"
      ],
      "id": "1adf4f5d-c92a-4bc4-8d36-65d8441b7b88"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "fnames = ['./results/'+l for l in os.listdir('./results/') if l[0] != '.']\n",
        "fnames "
      ],
      "id": "7993dc3e-eb81-478c-a79b-3eccbb72e189"
    },
    {
      "cell_type": "markdown",
      "metadata": {},
      "source": [
        "## RSLT"
      ],
      "id": "4fe6a844-1c56-48ba-8871-52e8f55cee01"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "df_ = pd.concat([pd.read_csv(fname) for fname in fnames]).reset_index(drop=True)"
      ],
      "id": "6df56734-658c-46c9-b241-212e12579497"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "df_"
      ],
      "id": "302cbc8b-3c1f-43dc-ae38-438e4e8f497b"
    },
    {
      "cell_type": "code",
      "execution_count": null,
      "metadata": {},
      "outputs": [],
      "source": [
        "df_.groupby(by=['model','method']).agg({'mse':[np.mean, np.std]})"
      ],
      "id": "da0b42b1-cf79-43bf-827a-bcf0439e41b2"
    }
  ],
  "nbformat": 4,
  "nbformat_minor": 5,
  "metadata": {
    "kernelspec": {
      "name": "python3",
      "display_name": "Python 3",
      "language": "python"
    },
    "language_info": {
      "name": "python",
      "codemirror_mode": {
        "name": "ipython",
        "version": "3"
      },
      "file_extension": ".py",
      "mimetype": "text/x-python",
      "nbconvert_exporter": "python",
      "pygments_lexer": "ipython3",
      "version": "3.8.16"
    }
  }
}